package com.pdaaphm.biz.service;

import java.io.IOException;
import java.util.List;
import java.util.Map;

import com.pdaaphm.biz.domain.Algorithm;
import com.pdaaphm.biz.dto.AlgorithmDTO;
import org.springframework.scheduling.annotation.Async;

/**
 * 算法Service接口
 *
 * @author xlk
 * @date 2023-07-26
 */
public interface IAlgorithmService
{
    /**
     * 查询算法
     *
     * @param id 算法主键
     * @return 算法
     */
    public Algorithm selectAlgorithmById(Long id);

    /**
     * 查询算法列表
     *
     * @param algorithm 算法
     * @return 算法集合
     */
    public List<Algorithm> selectAlgorithmList(Algorithm algorithm);

    /**
     * 新增算法
     *
     * @param algorithm 算法
     * @return 结果
     */
    public int insertAlgorithm(Algorithm algorithm);

    /**
     * 新增算法
     *
     * @param algorithmDto 算法
     * @return 结果
     */
    public int addOrUpdateDto(AlgorithmDTO algorithmDto);

    /**
     * 修改算法
     *
     * @param algorithm 算法
     * @return 结果
     */
    public int updateAlgorithm(Algorithm algorithm);

    /**
     * 批量删除算法
     *
     * @param ids 需要删除的算法主键集合
     * @return 结果
     */
    public int deleteAlgorithmByIds(Long[] ids);

    /**
     * 删除算法信息
     *
     * @param id 算法主键
     * @return 结果
     */
    public int deleteAlgorithmById(Long id);

    public List<Map> getOption();

    /**
     * 查询算法对象
     *
     * @param id 算法主键
     * @return 算法
     */
    public AlgorithmDTO getAlgorithmDto(Long id);

    public void runAlgorithms(Long id) throws IOException;
}
