package com.pdaaphm.biz.service.impl;

import java.util.List;
import java.util.Map;

import com.pdaaphm.biz.mapper.AlgorithmSubTypeMapper;
import com.pdaaphm.biz.mapper.SubAlgorithmMapper;
import com.pdaaphm.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.pdaaphm.biz.mapper.FileMapper;
import com.pdaaphm.biz.domain.File;
import com.pdaaphm.biz.service.IFileService;

/**
 * 文件Service业务层处理
 *
 * @author xlk
 * @date 2023-07-26
 */
@Service
public class FileServiceImpl implements IFileService {
    @Autowired
    private FileMapper fileMapper;

    @Autowired
    private SubAlgorithmMapper subAlgorithmMapper;

    /**
     * 查询文件
     *
     * @param id 文件主键
     * @return 文件
     */
    @Override
    public File selectFileById(Long id) {
        return fileMapper.selectFileById(id);
    }

    /**
     * 查询文件列表
     *
     * @param file 文件
     * @return 文件
     */
    @Override
    public List<File> selectFileList(File file) {
        return fileMapper.selectFileList(file);
    }

    @Override
    public List<File> selectInputFileList(File file) {
        file.setType(File.inputFlag);
        return fileMapper.selectInputFileList(file);
    }

    /**
     * 新增文件
     *
     * @param file 文件
     * @return 结果
     */
    @Override
    public int insertFile(File file) {
        file.setCreateTime(DateUtils.getNowDate());
        return fileMapper.insertFile(file);
    }

    /**
     * 修改文件
     *
     * @param file 文件
     * @return 结果
     */
    @Override
    public int updateFile(File file) {
        file.setUpdateTime(DateUtils.getNowDate());
        return fileMapper.updateFile(file);
    }

    /**
     * 批量删除文件
     *
     * @param ids 需要删除的文件主键
     * @return 结果
     */
    @Override
    public int deleteFileByIds(Long[] ids) {
        for (Long id : ids) {
            subAlgorithmMapper.deleteSubAlgorithmByUploadId(id);
        }
        return fileMapper.deleteFileByIds(ids);
    }

    /**
     * 删除文件信息
     *
     * @param id 文件主键
     * @return 结果
     */
    @Override
    public int deleteFileById(Long id) {
        subAlgorithmMapper.deleteSubAlgorithmByUploadId(id);
        return fileMapper.deleteFileById(id);
    }

    @Override
    public List<Map> getOption() {
        return fileMapper.getOption();
    }

    @Override
    public List<Map> getInputOption(Long subTypeId) {
        return fileMapper.getInputOption(subTypeId);
    }
}
